from typing import Optional, Pattern, Self
from collections import UserList

from .Lyric_Time_tab import Lyric_Time_tab
from .Lyric_character import Lyric_character
import re


class Lyric_line_content(UserList):
    """
    歌词行内容类
    接受字符串，预先分割
    根据时间，字符，时间，字符，时间，字符 匹配
    分离每个时间，和字符
    如果该字符没有时间，则填None
    然后，将每个时间和字符组合成一个Lrc_character对象
    最后，将每个Lrc_character对象添加到有序字典中，以字符为键，读音（振假名用）为值
    有宽松匹配模式，还有严格匹配模式
    [[Lrc_char obj, [[读音]， 2]], # 2表示显示，两个字符一个读音
    [Lrc_char obj, [""， None]], # None表示前面有读音，是多个字符一个读音
    [Lrc_char obj, [[读音]， 1]] # 1表示显示字符长度个读音
    [Lrc_char obj, [[Any str]， 0]] # 0表示不显示
    没有pronunciation直接None，防止无限递归
    ...
    ]
    读音是一个列表，因为一个字符可能有多个读音，用Lrc_character对象储存（可以包含时间）
    读音也可以填入Lrc_character(None)，表示不显示东西
    """

    def __init__(self, line: str,
                 pronunciation_list: Optional[list[Optional[Self], int]] = None,
                 separation_mode: str = "normal"):
        # 有序字典，超类重写
        super().__init__()

        self.pronunciation_list = pronunciation_list
        self.original_line: str = line

        # 预分离字符串
        self._time_char_list: list[list[str, str]] = Lyric_line_content.split_line_to_time_and_char(line,
                                                                                                    separation_mode)
        # 时间字符对象列表
        self.time_char_object_list: list[Lyric_character] = []

        # 先转为Time_tab对象
        # 再转为Lrc_character对象
        # 再添加到有序字典中
        for time_char in self._time_char_list:
            each_time_tab_object: Lyric_Time_tab = Lyric_Time_tab(time_char[0],
                                                                  separation_mode)

            each_char_object: Lyric_character = Lyric_character(time_char[1],
                                                                each_time_tab_object)

            self.time_char_object_list.append(each_char_object)

        self.data = self.time_char_object_list

        # 读音列表
        self.update_pronunciation_list(pronunciation_list)


    def __add__(self, other):
        if isinstance(other, Lyric_line_content):
            # 判断是否有发音列表
            if self.pronunciation_list is None and other.pronunciation_list is None:
                return Lyric_line_content(self.original_line + other.original_line, None)
            elif self.pronunciation_list is None:
                return Lyric_line_content(self.original_line + other.original_line, other.pronunciation_list)
            elif other.pronunciation_list is None:
                return Lyric_line_content(self.original_line + other.original_line, self.pronunciation_list)
            else:
                return Lyric_line_content(self.original_line + other.original_line,
                                          self.pronunciation_list.append(other.pronunciation_list))

        elif isinstance(other, str):
            return Lyric_line_content(self.original_line + other, self.pronunciation_list.append(["", 0]))

        else:
            raise TypeError("Unsupported operand type(s) for +: 'Lyric_line_content' and '{}'".format(type(other)))

    def __radd__(self, other):
        if isinstance(other, str):
            return Lyric_line_content(other + self.original_line, self.pronunciation_list.append(["", 0]))

        else:
            raise TypeError("Unsupported operand type(s) for +: 'Lyric_line_content' and '{}'".format(type(other)))

    def __str__(self):
        return self.original_line

    def __repr__(self):
        return self.original_line

    def isspace(self):
        return self.original_line.isspace()

    """
    对应发音和自身字符
    """
    def update_pronunciation_list(self, pronunciation_list: list[list[Self, int]]) -> Self:
        # 更新发音列表
        # None表示全段没有读音
        self.pronunciation_list: Optional[
            list[
                Optional[Self],
                Optional[int]
            ]
        ] = pronunciation_list

        # 验错区
        # 开始对应发音（振假名）和字符
        # 如果没有发音，则填入None
        # 但是先检查列表是否等长，如果不等长，就报错
        if self.pronunciation_list is not None:
            if len(self.pronunciation_list) != len(self.time_char_object_list):
                raise ValueError("Length of pronunciation_list and time_char_object_list are not equal")

            # 测试合法性
            elif self.check_pronunciation_list_validity() is False:
                raise ValueError("Invalid pronunciation_list")

        # 处理区
        # 对应发音和字符
        if self.pronunciation_list is None:
            # 初始化
            self.pronunciation_list = []
            # 循环字符对象列表
            for each_char_object in self.time_char_object_list:
                # print(each_char_object)
                # 对应字符和发音
                self.pronunciation_list.append(["", 0])
        else:
            # 直接对应
            self.pronunciation_list = self.pronunciation_list

        return self

    """
    格式：
    [[Self,  2], # 2表示显示，两个字符一个读音
    [Self,  None], # None表示前面有读音，是多个字符一个读音
    [Self,  1],# 1表示显示字符长度个读音
    [Self， 0]] # 0表示不显示
    测试完毕
    """
    @staticmethod
    def check_pronunciation_list_validity_staticmethod(pronunciation_list: list[Optional[Self], int]) -> bool:
        """
        检查是否符合格式：
        [[Self,  2], # 2表示显示，两个字符一个读音
        [Self,  None], # None表示前面有读音，是多个字符一个读音
        [Self,  1],# 1表示显示字符长度个读音
        [Self， 0]] # 0表示不显示

        :param pronunciation_list: 读音列表
        :return: 是否合法
        """
        # 如果列表为空，返回False
        if not pronunciation_list:
            return False

        # 遍历列表中的每一项
        for i, item in enumerate(pronunciation_list):
            # 如果项不是一个长度为2的列表，返回False
            if not isinstance(item, list) or len(item) != 2:
                return False

            # 如果项的第一个元素不是Lyric_line_content，返回False
            if not isinstance(item[0], Lyric_line_content):
                return False

            # 如果项的第二个元素不是一个整数或None，返回False
            if not isinstance(item[1], (int, type(None))):
                return False

            # 如果项的第二个元素是负数或大于剩余列表长度，返回False
            if (item[1] is not None) and (item[1] < 0 or item[1] > len(pronunciation_list) - i - 1):
                return False

            # 如果项的第二个元素大于1，表示占用了后面的字符，检查后面的字符是否为[Self, None]或[None, None]
            # 0 表示不显示
            if (item[1] is not None) and (item[1] > 1):
                for j in range(i + 1, i + item[1]):
                    if not (isinstance(pronunciation_list[j][0], Lyric_line_content)
                            and pronunciation_list[j][1] is None):
                        return False

            # 如果项的第二个元素是0，表示不显示，跳过后续检查
            if item[1] == 0:
                continue

        # 如果所有项都通过了检查，返回True
        return True

    """
    实例方法
    """
    def check_pronunciation_list_validity(self) -> bool:
        """
        检查是否符合格式：
        [[Self,  2], # 2表示显示，两个字符一个读音
        [Self,  None], # None表示前面有读音，是多个字符一个读音
        [Self,  1],# 1表示显示字符长度个读音
        [Self， 0]] # 0表示不显示

        :return: 是否合法
        """
        return Lyric_line_content.check_pronunciation_list_validity_staticmethod(self.pronunciation_list)


    """
    预分离字符串
    输入格式：时间+字符串+时间+字符串+时间+字符串...
    返回格式：[[时间， 单个字符], [时间， 单个字符], [时间， 单个字符]...]
    没有时间的字符，时间填None，没有字符的时间，字符填None
    """
    @staticmethod
    def split_line_to_time_and_char(line: str,
                                    separation_mode: str = "strict"
                                    ) -> list[list[str, str]]:
        """
        预分离字符串
        输入格式：时间+字符串+时间+字符串+时间+字符串...
        返回格式：[[时间， 单个字符], [时间， 单个字符], [时间， 单个字符]...]
        没有时间的字符，时间填None，没有字符的时间，字符填None

        :param line: 字符串
        :param separation_mode: 分离模式
        :return: 分离后的字符串列表
        """

        SEPARATION_PATTERN: Pattern

        # 严格模式 根据Time_tab类的正则表达式分离
        if separation_mode == "strict":
            SEPARATION_PATTERN = Lyric_Time_tab.TIME_TAB_EACH_WORD_STRICT_REGREX

        # 普通模式 根据Time_tab类的正则表达式分离
        elif separation_mode == "normal":
            SEPARATION_PATTERN = Lyric_Time_tab.TIME_TAB_EACH_WORD_NORMAL_REGREX

        # 宽松模式 根据Time_tab类的正则表达式分离
        elif separation_mode == "loose":
            SEPARATION_PATTERN = Lyric_Time_tab.TIME_TAB_EACH_WORD_LOOSE_REGREX

        # 非常宽松模式 根据Time_tab类的正则表达式分离
        elif separation_mode == "very_loose":
            SEPARATION_PATTERN = Lyric_Time_tab.TIME_TAB_EACH_WORD_VERY_LOOSE_REGREX

        # 输入不合法
        else:
            raise ValueError("separation_mode must be strict, normal, loose or very_loose")

        # 初始化结果列表
        result: list[list[str, str]] = []

        # 初始化上一个匹配的结束位置为 0
        prev_end = 0

        # 初始化当前时间标签为 None
        current_time_tag = None

        # 使用正则表达式匹配时间标签和字符
        for matched_char_time_tab in re.finditer(SEPARATION_PATTERN, line):
            # 获取匹配的起始和结束位置
            start, end = matched_char_time_tab.span()

            result.append([current_time_tag, line[prev_end]])

            # 如果匹配的起始位置大于上一个匹配的结束位置，说明两个匹配之间有其他字符
            if start > (prev_end + 1):
                for char in line[prev_end + 1:start]:
                    result.append([None, char])

            # 更新当前时间标签
            current_time_tag = matched_char_time_tab.group()

            # 更新上一个匹配的结束位置
            prev_end = end

        # 如果最后一个匹配的结束位置不是字符串的末尾，说明最后一个匹配之后还有其他字符
        if prev_end < len(line):
            result.append([current_time_tag, line[prev_end]])
            # 如果只有一个字符，那么直接添加[时间标签, char]
            if prev_end == (len(line) - 1):
                pass
            # 如果有多个字符，那么添加[时间标签, char]，然后添加[None, char]
            else:
                for char in line[prev_end + 1:]:
                    result.append([None, char])

        # 如果最后一个匹配的结束位置是字符串的末尾，说明最后一个匹配之后没有其他字符，用""代替表示 空字符串
        else:
            result.append([current_time_tag, ""])

        return result

    """
    将日语kana标签转换为发音列表
    """

    """
    character_indexes: list[list[str | Self, int]] 字符，索引
    character_pronunciations: list[list[str, int]] 字符发音，占用长度
    length: int 总长度
    """
    @staticmethod
    def extend_pronunciation_list(character_indexes: list[list[str | Self, int]],
                                  character_pronunciations: list[list[str, int]],
                                  length: int) -> list[list[Self | str, int]]:

        # 提取所有索引, 不考虑乱序
        character_indexes_only: list[int] = [character_index[1] for character_index in character_indexes]
        # 判断是否有重复的index，判断是否有index超出范围，判断是否等长
        if len(character_indexes) != sum(character_indexes_only):
            raise ValueError("character_indexes and character_pronunciations must have the same length")
        elif len(set([character_index[1] for character_index in character_indexes])) != len(character_indexes):
            raise ValueError("character_indexes must not have duplicate indexes")
        elif max([character_index[1] for character_index in character_indexes]) >= length:
            raise ValueError("character_indexes must not have index out of range")

        # 初始化
        output_complete_pronunciation_list: list[Self, int] = []

        # 没有发音的默认情况
        no_pronunciation_character = ["", 0]
        # 被占位的情况
        occupied_pronunciation_character = ["", None]

        time_jump: int = 0

        for char_index in range(length):

            # 表示被占用
            if time_jump != 0:
                time_jump -= 1
                # append 被占用的情况
                output_complete_pronunciation_list.append(occupied_pronunciation_character)
                continue

            # 初始化
            # 依次 对号入座
            if char_index == character_indexes_only[0]:
                each_character_pronunciation: list[str, int] = character_pronunciations[0]

                # 设置之后的 n-1（本字符已经添加） 被占位
                time_jump: int = each_character_pronunciation[1] - 1

                # append 一般字符
                output_complete_pronunciation_list.append(each_character_pronunciation)

            # 另外，就是没有发音
            else:
                output_complete_pronunciation_list.append(no_pronunciation_character)

        return output_complete_pronunciation_list




    '''
    利用方法Lyric_character。is_chinese_or_chu_nom_or_chinese_radical_staticmethod
    获取一行中所有的汉字，喃字，以及位置，返回一个列表
    '''

    @staticmethod
    def get_all_chinese_and_chu_nom_and_chinese_radical_staticmethod(line: str) -> list[list[str, int]]:
        """
        获取一行中所有的汉字，喃字，以及位置，返回一个列表
        Get all Chinese characters, Chu Nom characters, and their positions in a line, and return a list

        :param line: 一行歌词 A line of lyrics
        :return: 一个列表，列表中的每个元素都是一个列表，列表中的第一个元素是汉字或者喃字，第二个元素是汉字或者喃字的位置
         A list, each element in the list is a list,
         the first element in the list is a Chinese character or Chu Nom character,
         and the second element is the position of the Chinese character or Chu Nom character

        :rtype: list[list[str, int]]
        """

        # 初始化结果列表
        result = []
        # 逐字检查
        for index, char in enumerate(line):
            # 如果是汉字或者喃字，那么添加到结果列表中
            if Lyric_character.is_chinese_or_chu_nom_or_chinese_radical_staticmethod(char):
                result.append([char, index])

        # 返回结果
        return result

    '''
    利用方法Lyric_character。is_chinese_or_chu_nom_or_chinese_radical_staticmethod
    获取一行中所有的汉字，喃字，以及位置，返回一个列表
    利用自身的静态方法
    '''

    def get_all_chinese_and_chu_nom_and_chinese_radical(self):
        """
        获取一行中所有的汉字，喃字，以及位置，返回一个列表
        Get all Chinese characters, Chu Nom characters, and their positions in a line, and return a list

        :return: 一个列表，列表中的每个元素都是一个列表，列表中的第一个元素是汉字或者喃字，第二个元素是汉字或者喃字的位置
         A list, each element in the list is a list,
         the first element in the list is a Chinese character or Chu Nom character,
         and the second element is the position of the Chinese character or Chu Nom character

        :rtype: list[list[str, int]]
        """

        # 初始化结果列表
        result: list[list[str, int]] = []

        # 提取字符串
        for index, character in enumerate(self):
            if Lyric_character.is_chinese_or_chu_nom_or_chinese_radical_staticmethod(str(character)):
                result.append([character, index])

        return result

    def format_content(self,
                       len_of_millisecond_output: int = 2,
                       seperator: tuple[str, str] = (":", "."),
                       bracket: tuple[str, str] = ("<", ">")
                       ) -> str:
        output_str: str = ""
        for each_lyric_character in self:
            each_lyric_character: Lyric_character

            each_time_tab = each_lyric_character.time_tab
            each_time_tab_str = each_time_tab.convert_to_time_tab(len_of_millisecond_output=len_of_millisecond_output,
                                                                  seperator=seperator,
                                                                  brackets=bracket)

            output_str += (str(each_time_tab_str) + str(each_lyric_character))

        return output_str


    def get_kana_tag(self) -> str:
        """
        获取假名标签 形式的字符串
        Get kana tag type string

        :return: 假名标签
         Kana tag

        :rtype: str
        """
        # 获取读音列表
        pronunciation_list = self.pronunciation_list

        output_str: str = ""

        # 逐个遍历
        for each_pronunciation in pronunciation_list:
            each_pronunciation: list[str, int]

            # 先看占位符
            if each_pronunciation[1] is None:
                continue
            # 再看读音
            else:
                if each_pronunciation[0] is None:
                    # 不可以是None，否则会报警告
                    SyntaxWarning("Pronunciation cannot be None, maybe use \"\" to  substitute it")
                    continue

                # 另外的情况就加到结果字符串中
                # 以，长度+读音的形式
                output_str += (str(each_pronunciation[1]) + str(each_pronunciation[0]))

        return output_str


if __name__ == '__main__':
    # test_str: str = " <00:0.0>あ <00:01:000>い <00:02:000>う <00:03:000>え <00:04:000>お1 <00:04:000>"
    # print(Lyric_line_content.split_line_to_time_and_char(test_str, "strict"))
    # print(Lyric_line_content.split_line_to_time_and_char(test_str, "normal"))
    # print(Lyric_line_content.split_line_to_time_and_char(test_str, "loose"))
    # print(Lyric_line_content.split_line_to_time_and_char(test_str, "very_loose"))

    # 测试Lyric_line_content.get_all_chinese_and_chu_nom_and_chinese_radical
    test_str: str = " <00:0.0>あNi你他 <00:01:000>い  <00:02:000>う <00:03:000>え <00:04:000>お1 <00:04:000>"
    Lrc_line_content_obj = Lyric_line_content(test_str, separation_mode="very_loose")
    print(Lrc_line_content_obj.get_all_chinese_and_chu_nom_and_chinese_radical())
    print(Lrc_line_content_obj.time_char_object_list)
    print(Lrc_line_content_obj.pronunciation_list)

    # 测试加法
    # print(a := Lyric_line_content("a") + Lyric_line_content("b"))

    # print(a["a"])

    c = Lyric_line_content("")
    print(c.time_char_object_list)
    print(c.pronunciation_list)
    print(c.data)
    output = Lrc_line_content_obj.format_content()
    print(output)


import re
import warnings
from typing import Any, Iterator, List
from typing import Optional, Self
from typing import Union
from typing import Pattern, Match
from typing import Callable
from .Lyric_Time_tab import Lyric_Time_tab
from .Lyric_line_content import Lyric_line_content
from .Lyric_line import Lyric_line


class Lyric_file:
    """
    中文：\n
    初始化函数，接受一个参数，参数类型为str或者list，分别对应文件内容和文件内容的列表形式 \n
    \n
    str 为文件内容 \n
    list 为文件内容的列表形式，每个元素为一行歌词

    English: \n
    Initialization function, accepts one parameter,
    the parameter type is str or list,
    which corresponds to the file content
    and the list form of the file content respectively \n
    \n
    str is the file content \n
    list is the list form of the file content, each element is a line of lyrics

    :param lrc_content: Union[str, list]
    :return: None
    """

    """
    同时兼容文件和文件内容
    其实没有必要，可以用另外一个函数生成饭后返回实例化后的本类
    本类专注于处理文件内容，而不是文件本身
    """

    # ==================== 正则表达式区 ====================
    # 信息标签的正则表达式
    # 必须是字母开头，后面随意字符，直到冒号
    # 冒号作为分隔符，然后是任意字符
    # 括号可以缺失，但是冒号不能缺失，括号必须是中括号
    # groups: 左括号，标签tag，冒号，内容tag_content，右括号
    # 例如：[ar:周杰伦]
    INFORMATION_TAG_REGEX: Pattern = re.compile(r"^(\[)?(?P<tag>[a-zA-Z].*):(?P<tag_content>.*)(])?$")

    # 正式歌词的正则表达式
    # 中括号可以缺失，但是冒号不能缺失，起始标签的括号必须是中括号
    # 分为两组
    # groups: (左括号，分，冒号，秒，冒号或者点，毫秒，右括号)[时间标签组]，歌词内容[歌词内容组]
    # 例如：[00:00.00]歌词内容
    LRC_CONTENT_REGEX: Pattern = re.compile(r"^(?P<time_tab>((\[)?(\d+)(:)(\d+)([.:])(\d*)(])?)*)"
                                            r"(?P<lrc_content>.*)$")

    # 标准格式
    INFORMATION_TAG_REGEX_STANDARD: Pattern = re.compile(r"^(\[)(?P<tag>[a-zA-Z].*):(?P<tag_content>.*)(])$")
    LRC_CONTENT_REGEX_STANDARD: Pattern = re.compile(r"^(?P<time_tab>((\[)(\d{2})(:)(\d{2})(\.)(\d{2})(])?)*)"
                                                     r"(?P<lrc_content>.*)$")

    EACH_PROUNUNCIATION_GROUP_IN_KANA_REGEX: Pattern = re.compile(r"(?P<all_group>"
                                                                  r"(?P<character_length>\d)"
                                                                  r"(?P<pronounciation>\D*)"
                                                                  r")")

    # ==================== 正则表达式区结束 ====================

    def __init__(self, lrc_content: Union[str, list], mode: str = "normal") -> None:
        """
        中文：\n
        初始化函数，接受一个参数，参数类型为str或者list，分别对应文件内容和文件内容的列表形式 \n
        \n
        str 为文件内容 \n
        list 为文件内容的列表形式，每个元素为一行歌词

        English: \n
        Initialization function, accepts one parameter,
        the parameter type is str or list,
        which corresponds to the file content
        and the list form of the file content respectively \n
        \n
        str is the file content \n
        list is the list form of the file content, each element is a line of lyrics

        :param lrc_content: Union[str, list]
        :return: None
        """

        # ==================== 歌词的内容属性区 ====================

        # 一级列表，每个元素为一行歌词，原始歌词备份区
        self.lrc_lines_primary_initial: list[str] = []
        # 二级列表，每个元素为一行歌词，每行歌词为一个列表，列表第一个元素为时间标签类或者None（表示只有内容），第二个元素为歌词内容(也是一个类)
        # 之后所有的操作都是基于这个列表，这个列表是最终的歌词内容
        self.lrc_lines_secondary: list[Lyric_line] = []

        # ==================== 歌词的内容属性区结束 ====================

        # ==================== 歌词的信息属性区 ====================
        # tag 初始化，考虑后期封装
        # 基本 tag 的属性
        # 设定TAG初始值为None
        self.artist: Optional[str] = None
        self.album: Optional[str] = None
        self.lyric_writer: Optional[str] = None
        self.lrc_file_writer: Optional[str] = None
        self.length: Optional[str] = None
        # self.length_int: Optional[int] = None
        self.offset: Optional[str] = None
        self.creating_software: Optional[str] = None
        self.title: Optional[str] = None
        self.version: Optional[str] = None

        # 日语歌词专用，不确定具体含义
        self.kana: Optional[str] = None

        # 非标准 tag 字典
        self.nonstandard_tag_dict: dict[str, str] = {}

        # 提前建立所有标签的列表，字典可能会乱序，为后面json做准备
        self.tag_list = [
            "artist",
            "album",
            "lyric_writer",
            "lrc_file_writer",
            "length",
            "offset",
            "creating_software",
            "title",
            "version",
            "kana"
        ]
        # 字典，方便查找
        # 标签，标签的含义（也是属性名）
        self.tag_dict: Optional[dict[str, str]] = {'ar': 'artist',
                                                   'al': 'album',
                                                   'au': 'lyric_writer',
                                                   'by': 'lrc_file_writer',
                                                   'length': 'length',
                                                   'offset': 'offset',
                                                   're': 'creating_software',
                                                   'ti': 'title',
                                                   've': 'version',
                                                   'kana': 'kana'  # 日语歌词专用，不确定具体含义
                                                   }

        # ==================== 歌词的信息属性区结束 ====================

        # ==================== 歌词总属性区 ====================

        self.mode: str = mode

        # 歌词文件的编码，默认是None，即不指定编码，以纯文本的形式读取
        self.lrc_encoding: Optional[str] = None

        # 获取是否为拓展LRC格式，默认为False
        self.whether_extension: bool = False

        # ==================== 歌词总属性区结束 ====================

        # ==================== 内部设置属性区 ====================

        # 定义__next__返回下一个的初始化[clc]
        # 用于自我迭代
        self.__next__index: int = 0

        # ==================== 内部设置属性区结束 ====================

        # ==================== 预分离处理 ====================
        # 预分离处理
        # 调用预分离处理函数
        self.__pre_separation(lrc_content, mode)

        # ==================== 预分离处理结束 ====================

    def __pre_separation(self, lrc_content: Union[str, list], mode: str) -> None:
        """
        中文：
        预分离处理，根据接受的参数类型，分别进行处理

        English:
        Pre-separation processing, according to the type of parameters received, respectively processing

        :param lrc_content: Union[str, list]
        :return: None
        """

        # 是字符串，按行分割，去除空行（含只有空白字符的行）
        if isinstance(lrc_content, str):
            # 按行分割
            self.lrc_lines_primary_initial = lrc_content.splitlines()
            # 去除空行
            self.lrc_lines_primary_initial = [line for line in self.lrc_lines_primary_initial if line.strip()]
        # 是列表，直接赋值，但是依然要去除空行（含只有空白字符的行）
        elif isinstance(lrc_content, list):
            self.lrc_lines_primary_initial = [line for line in lrc_content if line.strip()]
        # 不是字符串也不是列表，抛出异常
        else:
            raise TypeError("lrc_content must be str or list divided by lines")

        # 调用信息分离处理函数，分离歌词内容和歌词信息
        # 传入参数为歌词内容的列表
        self.__lrc_information_content_separation(self.lrc_lines_primary_initial, mode)

        # 合并跨行歌词
        self.merge_cross_line_lyrics()

    # 歌词内容分离处理函数，分离歌词内容和歌词信息
    # 传入参数为歌词内容的列表
    def __lrc_information_content_separation(self, lrc_lines: list[str], mode: str) -> None:
        """
        中文：
        歌词内容分离处理函数，分离歌词内容和歌词信息
        判断是否为扩展LRC格式

        English:
        Lyric content separation processing function, separate lyric content and lyric information
        Determine whether it is an extended LRC format

        :param lrc_lines: list of lyric content, divided by lines
        :return: None
        """
        # 逐行处理
        for line in lrc_lines:
            # 先判断是否为扩展LRC格式
            # 设置属性
            if self.whether_extension is False:
                # 如果是扩展LRC格式
                if "<" in line or ">" in line:
                    self.whether_extension = True

            # 按照正则表达式匹配，逐行匹配
            # 如果是歌词信息
            if each_line_match := self.INFORMATION_TAG_REGEX.match(line):
                # 歌词信息处理函数
                self.__lrc_information_processing(each_line_match)
            # 其他默认为歌词内容
            # 如果是歌词内容，（其他内容可以选择是否合并到上一行歌词）
            else:
                # 歌词内容处理函数
                self.__lrc_content_processing(line, mode)

        # 最后处理
        # 考虑到涉及整个文件的内容
        # 处理kana
        pronunciation_each_line = self.extract_kana_tag()
        # 更新每行的读音
        self.update_pronunciation(pronunciation_each_line)

    # 歌词信息处理函数
    # 传入参数为歌词信息的字符串
    def __lrc_information_processing(self, each_line_match: Match) -> None:
        """
        中文：
        歌词信息处理函数

        English:
        Lyric information processing function

        :param each_line_match: 歌词信息的字符串 the string of lyric information
        :return: None
        """
        # 获取标签
        tag = each_line_match.group("tag")
        # 识别歌词信息的标签，确定是否为默认标签之一（字典的键），如果是，那么直接复制给对应的属性
        if tag in self.tag_dict.keys():
            # 获取属性名
            tag_name = self.tag_dict[tag]
            # 获取属性值
            tag_value = each_line_match.group("tag_content")
            # 赋值
            setattr(self, tag_name, tag_value)

        # 如果不是，那么就是自定义的标签，储存到一部字典中
        else:
            # 获取标签名
            tag_name = each_line_match.group("tag")
            # 获取标签值
            tag_value = each_line_match.group("tag_content")
            # 添加到字典中
            self.nonstandard_tag_dict[tag_name] = tag_value



    # 歌词内容处理函数
    # 传入参数为歌词内容的字符串
    def __lrc_content_processing(self, each_line: str, mode: str) -> None:
        """
        中文：
        每行歌词内容处理函数
        分离时间标签和歌词内容

        English:
        Each line of lyric content processing function
        Separate the time tag and lyric content

        :param each_line: 每行歌词内容的字符串 the string of each line of lyric content
        :return: None
        """
        # 用正则表达式匹配，获取时间和歌词（时间可能为空）
        each_line_match = self.LRC_CONTENT_REGEX.match(each_line)

        time: str | Optional[Lyric_Time_tab]
        # 获取时间
        time: str = each_line_match.group("time_tab")

        # 分离时间标签，可能有多个时间标签，所以是列表
        time_str_list: list[tuple[str]] = Lyric_Time_tab.TIME_TAB_EACH_LINE_VERY_LOOSE_REGREX.findall(time)

        # 因为有括号，所以是元组的列表，需要转为字符串的列表
        time_str_list: list[str] = ["".join(each_time_tab_element_tuple)
                                    for each_time_tab_element_tuple in time_str_list]

        # 获取歌词
        lrc: str = each_line_match.group("lrc_content")

        # 转为类
        lrc_obj: Lyric_line_content = Lyric_line_content(lrc,
                                                         separation_mode=mode)

        # 判断时间 是否为空
        if time_str_list:
            time_obj: Optional[list[Lyric_Time_tab]] = []
            # 转为类的列表
            for each_time in time_str_list:
                # 转为类
                each_time_obj: Optional[Lyric_Time_tab] = Lyric_Time_tab(each_time, mode)
                # 放入一级歌词列表中
                time_obj.append(each_time_obj)

        # 如果为空，转为 None
        else:
            time_obj: Optional[list[Lyric_Time_tab]] = None

        # 包装成类
        each_lyric_line_obj: Lyric_line = Lyric_line(time_obj, [lrc_obj])

        # 放入二级歌词列表中
        self.lrc_lines_secondary.append(each_lyric_line_obj)
        # 合并跨行歌词
        self.merge_cross_line_lyrics()

    # 合并跨行歌词
    @staticmethod
    def merge_cross_line_lyrics_static(secondary_lyric_list: list[Lyric_line]
                                       ) -> list[Any,
                                                 Lyric_line_content]:
        """
        中文：
        合并跨行歌词

        English:
        Merge cross-line lyrics

        :return: None
        """

        # 检测第一行是否有时间标签
        # 没有或者空白字符串，报错
        if secondary_lyric_list[0].time_tabs is None or secondary_lyric_list[0].time_tabs == []:
            raise ValueError("The first line of lyrics does not have a time tag")

        # 返回的列表
        return_list: list[Any, Lyric_line_content] = []

        # 逐行处理
        for each_line_list in secondary_lyric_list:
            # 如果时间标签 有内容，那么就是新的一行歌词
            if each_line_list[0]:
                # 添加到列表中
                return_list.append(each_line_list)
            # 如果时间标签 为空，那么就是上一行歌词的延续
            else:
                # 添加到上一行歌词的列表中
                return_list[-1][1] += each_line_list[1]

        return return_list

    # 合并跨行歌词
    # 实例方法
    def merge_cross_line_lyrics(self) -> Self:
        """
        中文：
        合并跨行歌词
        不更新一级列表

        English:
        Merge cross-line lyrics
        Do not update the primary list

        :return: Self
        """
        # 调用静态方法
        self.lrc_lines_secondary = self.merge_cross_line_lyrics_static(self.lrc_lines_secondary)

        # 这里不需要更新一级列表

        # 返回自身
        return self

    @classmethod
    def convert_primary_lyric_list_to_secondary_list_classmethod(cls,
                                                                 lrc_lines: str | list[str],
                                                                 mode: str
                                                                 ) -> list[list[Lyric_Time_tab,
                                                                                Lyric_line_content]]:
        """
            中文：
            在纯歌词内容的字符串或列表中，分离时间标签和歌词内容

            English:
            In the string or list of pure lyric content, separate the time tag and lyric content

            :param lrc_lines: 歌词内容的字符串 || the string of lyric content
            :param mode: 时间标签检查严格度，可选值为 "strict", "normal", "loose" 和 “very_loose”
                || The strictness of the time tag check,
                 the optional values are "strict", "normal", "loose" and "very_loose"
            :return: list[list[Lyric_Time_tab, [str]]]
        """
        # 新的空列表
        output_list: list[str | list[Lyric_Time_tab], Lyric_line_content]

        # 如果是字符串，按行分割，去除空行（含只有空白字符的行）
        if isinstance(lrc_lines, str):
            # 按行分割
            output_list = lrc_lines.splitlines()
            # 去除空行
            output_list = [line for line in output_list if line.strip()]
        # 是列表，直接赋值，但是依然要去除空行（含只有空白字符的行）
        elif isinstance(lrc_lines, list):
            output_list = [line for line in lrc_lines if line.strip()]
        # 不是字符串也不是列表，抛出异常
        else:
            raise TypeError("lrc_content must be str or list divided by lines")

        # 逐行处理
        for lrc_lines in output_list:
            # 用正则表达式匹配，获取时间和歌词（时间可能为空）
            each_line_match = cls.LRC_CONTENT_REGEX.match(lrc_lines)

            time: str | Optional[Lyric_Time_tab]
            # 获取时间
            time = each_line_match.group("time")
            # 获取歌词
            lrc: str = each_line_match.group("lrc")

            lrc: Lyric_line_content = Lyric_line_content(lrc)

            # 判断时间 是否为空
            if time:
                time = Lyric_Time_tab(time, mode)
            # 否则转为None
            else:
                time = None

            # 覆盖原项，放入new_list中
            lrc_lines = [time, lrc]
            output_list[output_list.index(lrc_lines)] = lrc_lines

        # 合并跨行歌词
        output_list = cls.merge_cross_line_lyrics_static(output_list)

        # 返回新的列表
        return output_list

    # 判断是否为标准格式
    '''
    比如
    [ti:松花江上]
    [ar:张寒晖]
    [al:张寒晖]
    [by:张寒晖]
    [offset:0]
    [00:00.00] 我的家
    [00:01.00] 在那东北松花江上
    [00:03.00] 我的家
    [00:04.00] 那里有森林煤矿
    ......
    '''

    @classmethod
    def judge_standard_form_classmethod(cls, lrc_content: Union[str, list[str]]) -> bool:
        """
        中文：\n
        判断是否为标准格式
        lrc_content参数可以是字符串，也可以是字符串列表，如果是字符串，那么先按照换行符分割成列表
        之后逐行判断

        English: \n
        Determine whether it is standard format
        The lrc_content parameter can be a string or a string list.
        If it is a string, it will be divided into a list according to the line break symbol first.
        Then judge line by line.

        For example \n
        [ti:松花江上]
        [ar:张寒晖]
        ...
        [00:00.00] 我的家
        [00:01.00] 在那东北松花江上
        [00:03.00] 我的家
        [00:04.00] 那里有森林煤矿
        ...

        :return: bool
        """

        # 如果是字符串，那么先按照换行符分割成列表
        if isinstance(lrc_content, str):
            lrc_content = lrc_content.splitlines()
            # 去除空行（包括空白字符行）
            lrc_content = [line for line in lrc_content if line.strip()]

        # 之后逐行判断
        for line in lrc_content:
            # 如果符合两个标准格式的正则表达式中的一个，那么就是标准格式
            if (cls.INFORMATION_TAG_REGEX_STANDARD.match(line)
                    or cls.LRC_CONTENT_REGEX_STANDARD.match(line)):
                continue
            # 否则就不是标准格式
            else:
                return False

        return True

    # 针对实例
    # 判断是否为标准格式
    def judge_standard_form(self) -> bool:
        """
        中文：\n
        判断是否为标准格式

        English: \n
        Determine whether it is standard format

        :return: bool
        """
        return Lyric_file.judge_standard_form_classmethod(self.lrc_lines_primary_initial)

    # 合并换行的歌词为连续的行（中间用\n分割），也可以指定分隔符
    # 第一行有时间标签，后面的行没有时间标签，直到下一个有时间标签的行
    '''
    例子：
    [00:00.00] 我的家
    在那东北松花江上
    [00:03.00] 我的家
    那里有森林煤矿
    变成：
    [00:00.00] 我的家\n在那东北松花江上
    [00:03.00] 我的家\n那里有森林煤矿
    '''

    # 默认分隔符为\n
    # 默认输入的是歌词内容的二级列表
    # 默认的第一行有时间标签，所以直接从第二行开始判断，忽略第一行
    @staticmethod
    def combine_lyric_separated_to_continuous_lines_static(input_lyric_lines: list[Lyric_line],
                                                           separator: str = "\n",
                                                           ) -> list[Lyric_line]:
        """
        中文：\n
        合并换行的歌词为连续的行（中间用\n分割），也可以指定分隔符。
        第一行有时间标签，后面的行没有时间标签，直到下一个有时间标签的行。

        English: \n
        Combine the lyrics separated by each_lyric_line_object breaks into continuous lines (separated by \n in the middle),
        or you can specify a separator.
        The first each_lyric_line_object has a time tag,
         and the following lines do not have a time tag,
         until a each_lyric_line_object with a time tag.

        For example: \n
        [00:00.00] 我的家
        在那东北松花江上
        [00:03.00] 我的家
        那里有森林煤矿
        Become:
        [00:00.00] 我的家\n在那东北松花江上
        [00:03.00] 我的家\n那里有森林煤矿

        :param input_lyric_lines: list[Lyric_line]
        :param separator: str
        :return: list[list[Lyric_Time_tab, str]]
        """

        # 新的列表
        output_list: list[Lyric_line] = []

        # 先确认第一行有时间标签或者是非空字符串
        if not input_lyric_lines[0].lyric_contents:
            raise ValueError("The first each_lyric_line_object of the input list"
                             " does not have a time tag or is an empty string.")

        # 遍历每一行
        for each_lyric_line_object in input_lyric_lines:
            each_lyric_line_object: Lyric_line
            # 已经封装成Lyric_Time_tab对象了
            # 直接调用加法即可

            # 如果是第一行，那么直接放入新列表
            if each_lyric_line_object == input_lyric_lines[0]:
                output_list.append(each_lyric_line_object)
            # 如果不是第一行
            else:
                # 如果这一行没有时间标签，那么就和前一行合并
                # 注意这里的时间标签可以是Lyric_Time_tab对象，也可以是字符串
                # 只要不是None就行或者不是空字符串就行
                if each_lyric_line_object.time_tabs is None:
                    # 直接字符串拼接
                    output_list[-1].lyric_contents[-1] = output_list[-1].lyric_contents[0] \
                                                         + separator \
                                                         + each_lyric_line_object.lyric_contents[0]

                # 如果这一行有时间标签，那么就直接放入新列表
                else:
                    output_list.append(each_lyric_line_object)

        # 返回新列表
        return output_list

    # 实例方法
    def combine_lyric_separated_to_continuous_lines(self,
                                                    separator: str = "\n",
                                                    ) -> list[Lyric_line]:
        """
        中文：\n
        合并换行的歌词为连续的行（中间用\n分割），也可以指定分隔符。
        第一行有时间标签，后面的行没有时间标签，直到下一个有时间标签的行。

        English: \n
        Combine the lyrics separated by line breaks into continuous lines (separated by \n in the middle),
        or you can specify a separator.
        The first line has a time tag,
        and the following lines do not have a time tag,
        until a line with a time tag.

        For example: \n
        [00:00.00] 我的家
        在那东北松花江上
        [00:03.00] 我的家
        那里有森林煤矿
        Become:
        [00:00.00] 我的家\n在那东北松花江上
        [00:03.00] 我的家\n那里有森林煤矿

        :param separator: str
        :return: list[list[Lyric_Time_tab | str, str]]
        """

        # 调用静态方法
        # 赋值回二级列表
        output = Lyric_file.combine_lyric_separated_to_continuous_lines_static(
            input_lyric_lines=self.lrc_lines_secondary,
            separator=separator
        )

        # 记得根据二级列表更新一级列表

        # 返回自身
        return output

    # 将歌词的二级列表转换为一级列表
    # 二级列表的每一行的第一个元素是时间标签，第二个元素是歌词内容
    @staticmethod
    def convert_secondary_lyric_list_to_primary_list_static(input_lyric_lines: list[Lyric_line],
                                                            len_of_millisecond_output: int = 2,
                                                            seperator_each_line: tuple[str, str] = (":", "."),
                                                            seperator_inline: tuple[str, str] = (":", ".")
                                                            ) -> list[str]:
        """
        中文：\n
        将歌词的二级列表转换为一级列表。

        English: \n
        Convert the secondary list of lyrics into a primary list.

        :param input_lyric_lines: list[list[Lyric_Time_tab | str, str]]
        原始的二级列表
        Original secondary list
        :param len_of_millisecond_output: int
        输出的毫秒长度
        The length of the output milliseconds
        :param seperator_each_line: tuple[str, str]
        每一行时间标签的分隔符
        Separator for each line
        :param seperator_inline: tuple[str, str]
        每一行内部时间标签的分隔符
        Separator inside each line


        :return: list[str]
        一级列表
        Primary list
        """

        # 新的列表
        output_list: list[str] = []

        # 先合并
        input_lyric_lines = Lyric_file.combine_lyric_separated_to_continuous_lines_static(
            input_lyric_lines=input_lyric_lines
        )

        # 遍历每一行
        for line in input_lyric_lines:
            each_line: str = line.format_output(len_of_millisecond_output=len_of_millisecond_output,
                                                seperator_each_line=seperator_each_line,
                                                seperator_inline=seperator_inline
                                                )

            # 放入新列表
            output_list.append(each_line)

        # 返回新列表
        return output_list

    # 计算所有的翻译时间标签
    # 参数有
    # 百分比： b - a 的百分之多少 + a = (就是翻译的时间标签)
    # 是否忽略空歌词，空歌词会给占位符[None,None]
    # 最后一行的计算上限（b - a），因为最后一行没有下一行，通常设定为0
    def calculate_all_translated_time_tab(self,
                                          percentage: float = 0.5,
                                          last_line_time_tab_limit: float = 0,
                                          ignore_empty_lyric: bool = False,
                                          ) -> list[
        list[Optional[Lyric_Time_tab],
             Lyric_line_content]
    ]:
        """
        中文：\n
        计算所有的翻译时间标签。

        English: \n
        Calculate all translated time tags.

        :param percentage: float
        :param ignore_empty_lyric: bool
        :param last_line_time_tab_limit: float
        :return: Self
        """

        # 看第一行是否有时间标签
        # 如果没有时间标签 或者 第一行时间标签为空白字符串，那么就抛出异常
        if self.lrc_lines_secondary[0][0] is None:
            raise ValueError("第一行没有时间标签，无法计算。")
        elif isinstance(self.lrc_lines_secondary[0][0], Lyric_Time_tab):
            if self.lrc_lines_secondary[0][0].time_tab.isspace():
                raise ValueError("第一行时间标签为空白字符串，无法计算。")

        # 输出
        output_list: list[list[Optional[Lyric_Time_tab],
                               Lyric_line_content]] = []

        first_line_after_nonempty_index: int = 1

        if ignore_empty_lyric:
            # 找到第一行非空歌词的行号
            # 所有遍历的行，都需要预先添加入返回列表
            for index, line in enumerate(self.lrc_lines_secondary):
                # 添加入返回列表
                output_list.append(line)
                # 第一行非空歌词的行号
                if not line[1].isspace():
                    first_line_after_nonempty_index = index
                    # 时间标签
                    previous_line_time_tab: Lyric_Time_tab = line[0]
                    break
            # 如果全是空歌词，那么就抛出异常
            else:
                raise ValueError("All lines are empty lyrics.")
        else:
            # 前一行的时间标签，初始化为第一行的时间标签
            previous_line_time_tab: Lyric_Time_tab = self.lrc_lines_secondary[0][0]
            output_list.append(self.lrc_lines_secondary[0])

        # 当前行歌词
        # current_line_lyric: Optional[Lyric_line_content] = self.lrc_lines_secondary[0][1]

        # 下一行的时间标签，初始化为None
        # next_line_time_tab: Optional[Lyric_Time_tab] = None

        # 遍历每一行
        # 从第二行开始
        for line in self.lrc_lines_secondary[first_line_after_nonempty_index:]:
            # 先看是否有时间标签
            # 如果没有时间标签 或者 时间标签为空白字符串，跳过，直到找到时间标签为止
            if self.lrc_lines_secondary[0][0] is None:
                continue
            elif isinstance(self.lrc_lines_secondary[0][0], Lyric_Time_tab):
                if self.lrc_lines_secondary[0][0].time_tab.isspace():
                    continue

            # 先赋值给 下一行的时间标签临时变量
            next_line_time_tab = line[0]

            # 忽略空歌词 并且 当前行歌词 不 为空白字符串
            # 执行添加翻译时间标签 操作
            if ignore_empty_lyric and not line[1].isspace():
                # 计算翻译时间标签
                # 结果是time stamp
                translated_time_tab: float = (previous_line_time_tab
                                              + (next_line_time_tab - previous_line_time_tab)
                                              * percentage)

                # 转成时间标签字符串
                translated_time_tab: str = Lyric_Time_tab.convert_time_stamp_to_time_tab_static(
                    time_stamp=translated_time_tab,
                )

                # 转为时间标签对象
                translated_time_tab: Lyric_Time_tab = Lyric_Time_tab(
                    tab=translated_time_tab,
                    mode=self.mode
                )

                # 加入输出列表 [时间标签对象，翻译歌词("")空字符串]
                output_list.append([translated_time_tab, Lyric_line_content("")])

            # 更新前一行的时间标签
            previous_line_time_tab = next_line_time_tab

        # 最后一行的时间标签
        # 涉及到 last_line_time_tab_limit
        # 如果 last_line_time_tab_limit = 0，那么就是最后一行的时间标签
        # 其他情况，（last_line_time_tab_limit
        # - 那么就是最后一行的时间标签）* percentage
        # 计算翻译时间标签
        # 结果是time stamp
        translated_time_tab: float = (previous_line_time_tab
                                      + (last_line_time_tab_limit - previous_line_time_tab)
                                      * percentage)

        # 转成时间标签字符串
        translated_time_tab: str = Lyric_Time_tab.convert_time_stamp_to_time_tab_static(
            time_stamp=translated_time_tab,
        )

        # 转为时间标签对象
        translated_time_tab: Lyric_Time_tab = Lyric_Time_tab(
            tab=translated_time_tab,
            mode=self.mode
        )

        # 加入输出列表 [时间标签对象，翻译歌词(其实默认是空字符串)]
        output_list.append([translated_time_tab, Lyric_line_content("")])

        # 返回输出列表
        return output_list

    # 计算所有的翻译时间标签，然后更新二级列表
    # 和上面的方法不同，这个方法会更新二级列表
    def calculate_all_translated_time_tab_and_update_secondary_lyric_list(self,
                                                                          percentage: float = 0.5,
                                                                          last_line_time_tab_limit: float = 0,
                                                                          ignore_empty_lyric: bool = False
                                                                          ) -> Self:
        """
        中文：\n
        计算所有的翻译时间标签，然后更新二级列表。

        English: \n
        Calculate all translated time tags and update the secondary list.

        :param percentage: float
        :param last_line_time_tab_limit: float
        :param ignore_empty_lyric: bool
        :return: Self
        """

        # 看第一行是否有时间标签
        # 如果没有时间标签 或者 第一行时间标签为空白字符串，那么就抛出异常
        if self.lrc_lines_secondary[0][0] is None:
            raise ValueError("第一行没有时间标签，无法计算。")
        elif isinstance(self.lrc_lines_secondary[0][0], Lyric_Time_tab):
            if self.lrc_lines_secondary[0][0].time_tab.isspace():
                raise ValueError("第一行时间标签为空白字符串，无法计算。")

        # 输出
        output_list: list[list[Optional[Lyric_Time_tab],
                               Lyric_line_content]] = []

        first_line_after_nonempty_index: int = 1

        if ignore_empty_lyric:
            # 找到第一行非空歌词的行号
            # 所有遍历的行，都需要预先添加入返回列表
            for index, line in enumerate(self.lrc_lines_secondary):
                # 添加入返回列表
                output_list.append(line)
                # 第一行非空歌词的行号
                if not line[1].isspace():
                    first_line_after_nonempty_index = index
                    previous_line_time_tab: Lyric_Time_tab = line[0]
                    break

            else:
                # 如果没有非空歌词，那么就返回空列表
                # 说明所有歌词都是空的
                raise ValueError("All lyrics are empty.")

        else:
            # 前一行的时间标签，初始化为第一行的时间标签
            previous_line_time_tab: Lyric_Time_tab = self.lrc_lines_secondary[0][0]
            output_list.append(self.lrc_lines_secondary[0])

        # 下一行的时间标签，初始化为None
        next_line_time_tab: Optional[Lyric_Time_tab]

        # 遍历每一行
        # 从第二行开始
        for line in self.lrc_lines_secondary[first_line_after_nonempty_index:]:
            # 先把原来的歌词加入输出列表，后面跟翻译歌词
            output_list.append(line)
            # 先看是否有时间标签
            # 如果没有时间标签 或者 时间标签为空白字符串，跳过，直到找到时间标签为止
            if self.lrc_lines_secondary[0][0] is None:
                continue
            elif isinstance(self.lrc_lines_secondary[0][0], Lyric_Time_tab):
                if self.lrc_lines_secondary[0][0].time_tab.isspace():
                    continue

            # 先赋值给 下一行的时间标签临时变量
            next_line_time_tab = line[0]

            if ignore_empty_lyric and line[1].isspace():
                #    计算翻译时间标签
                # 结果是time stamp
                translated_time_tab: float = (previous_line_time_tab
                                              + (next_line_time_tab - previous_line_time_tab)
                                              * percentage)

                # 转成时间标签字符串
                translated_time_tab: str = Lyric_Time_tab.convert_time_stamp_to_time_tab_static(
                    time_stamp=translated_time_tab,
                )

                # 转为时间标签对象
                translated_time_tab: Lyric_Time_tab = Lyric_Time_tab(
                    tab=translated_time_tab,
                    mode=self.mode
                )

                # 加入输出列表 [时间标签对象，翻译歌词("")空字符串]
                output_list.append([translated_time_tab, Lyric_line_content("")])

            # 更新前一行的时间标签
            previous_line_time_tab = next_line_time_tab

            # 更新当前行歌词
            # current_line_lyric = line[1]

        # 最后一行的时间标签
        # 涉及到 last_line_time_tab_limit
        # 如果 last_line_time_tab_limit = 0，那么就是最后一行的时间标签
        # 其他情况，（last_line_time_tab_limit
        # - 那么就是最后一行的时间标签）* percentage
        # 计算翻译时间标签
        # 结果是time stamp
        translated_time_tab: float = (previous_line_time_tab
                                      + (last_line_time_tab_limit - previous_line_time_tab)
                                      * percentage)

        # 转成时间标签字符串
        translated_time_tab: str = Lyric_Time_tab.convert_time_stamp_to_time_tab_static(
            time_stamp=translated_time_tab,
        )

        # 转为时间标签对象
        translated_time_tab: Lyric_Time_tab = Lyric_Time_tab(
            tab=translated_time_tab,
            mode=self.mode
        )

        # 加入输出列表 [时间标签对象，翻译歌词(其实默认是空字符串)]
        output_list.append([translated_time_tab, Lyric_line_content("")])

        self.lrc_lines_secondary = output_list

        # 返回输出列表
        return self

    # 合并时间标签，把相同的歌词合并
    def compress_time_tab(self):
        # [00:00.00]歌词
        # [00:00.01]歌词
        # [00:00.02]歌词
        # 合并成 [00:00.00][00:00.01][00:00.02]歌词

        # 输出列表
        output_list: list[Lyric_line] = []

        for current_index, current_line in enumerate(self.lrc_lines_secondary):
            # 如果是第一行，那么就直接加入
            if current_index == 0:
                output_list.append(current_line)

            # 接下来每一行 都与之前的每一行比较
            # 如果歌词相同，那么就合并时间标签
            # 如果歌词不同，那么就直接加入
            else:
                for each_previous_line in output_list:
                    if each_previous_line.whether_same_lyric(current_line):
                        # 合并时间标签
                        each_previous_line.time_tabs += current_line.time_tabs
                        # 因为是合并，所以不需要再继续比较了，会出现重复
                        break
                else:
                    # 如果没有合并，那么就直接加入
                    output_list.append(current_line)

        # 覆盖原来的列表
        self.lrc_lines_secondary = output_list

        # 返回自身
        return self

    # 解压时间标签，把多个时间标签拆分成多行
    def decompress_time_tab(self) -> Self:
        # 输出列表
        output_list: list[Lyric_line] = []

        # 遍历每一行
        # 调用函数解压
        for current_line in self.lrc_lines_secondary:
            output_list += current_line.decompress_time_tab()

        self.lrc_lines_secondary = sorted(output_list)

        # 返回自身
        return self

    '''
    # 普通输出
    def output(self) -> str:
        warnings.WarningMessage("Not implemented yet.")
        pass
    '''

    # 格式化输出
    def format_output(self,
                      len_of_millisecond_output: int = 2,
                      seperator_each_line: tuple[str, str] = (":", "."),
                      seperator_inline: tuple[str, str] = (":", ".")
                      ) -> str:

        # 用于输出字符串
        output_str: str = ""

        # 先添加信息
        if self.artist:
            output_str += "[ar:" + self.artist + "]\n"
        if self.album:
            output_str += "[al:" + self.album + "]\n"
        if self.title:
            output_str += "[ti:" + self.title + "]\n"
        if self.length:
            output_str += "[length:" + self.length + "]\n"
        if self.lyric_writer:
            output_str += "[au:" + self.lyric_writer + "]\n"
        if self.lrc_file_writer:
            output_str += "[by:" + self.lrc_file_writer + "]\n"
        if self.creating_software:
            output_str += "[re:" + self.creating_software + "]\n"
        if self.version:
            output_str += "[ve:" + self.version + "]\n"
        if self.kana:
            output_str += "[kana:" + self.get_total_kana_tag() + "]\n"

        # 对自定义标签，非标准进行输出
        # 名字在之前的列表中
        for each_customized_tag, value in self.nonstandard_tag_dict:
            each_customized_tag: str
            value: str
            output_str += "[" + each_customized_tag + ":" + value + "]\n"

        time_tab_str: str
        # 遍历每一行
        for each_lyric_line in self.lrc_lines_secondary:
            each_lyric_line_str = each_lyric_line.format_output(len_of_millisecond_output=len_of_millisecond_output,
                                                                seperator_each_line=seperator_each_line,
                                                                seperator_inline=seperator_inline)

            output_str += each_lyric_line_str + "\n"

        # 去除最后一个换行符
        output_str = output_str[:-1]

        return output_str


    def update_kana_tag(self) -> Self:
        self.kana = self.get_total_kana_tag()

    def shift_time(self,
                   minutes: int,
                   seconds: int,
                   milliseconds: int,
                   len_of_millisecond: int = 3
                   ) -> Self:
        for each_lyric_line in self.lrc_lines_secondary:
            each_lyric_line.shift_time(minutes=minutes,
                                       seconds=seconds,
                                       milliseconds=milliseconds,
                                       len_of_millisecond=len_of_millisecond)

        return self

    def get_all_chinese_and_chu_nom_and_chinese_radical_list_each_line(self) -> list[list[list[str, int]]]:
        output_list = []
        for each_lyric_line in self.lrc_lines_secondary:
            output_list.append(each_lyric_line.get_all_chinese_and_chu_nom_and_chinese_radical())

        return output_list


    def extract_kana_tag(self) -> list[list[list[Lyric_line_content, int]]]:
        CJKV_list = self.get_all_chinese_and_chu_nom_and_chinese_radical_list_each_line()

        # 统计每行有多少个CJKV字符
        CJKV_count_each_line: list[int] = [len(each_line) for each_line in CJKV_list]

        # kana分解
        kana_character_list: list[list[str, int]] = self.split_kana_tag_to_characters(self.kana)
        # 每一行的综合体
        kana_character_line_list: list[list[list[str, int]]] = []
        # 累加器
        clc: int = 0
        # 对应每一行
        for each_line_number in CJKV_count_each_line:
            # 从累加器开始，到累加器+当前行的CJKV字符数
            kana_character_line_list.append(kana_character_list[clc:clc + each_line_number])
            # 累加
            clc += each_line_number

        # 输出列表
        # 输出的长发音列表，发音列表(行的长度，就是那个[])
        output_list: list[list[list[Lyric_line_content, int]]] = []
        # 对应每一行
        for each_line_CJKV_index, \
            each_line_pronunciation_list, \
            character_number_each_line \
                in zip(CJKV_list,
                       kana_character_line_list,
                       CJKV_count_each_line):

            # 直接调用方法
            each_line_pronunciation_full_list:  list[list[Lyric_line_content, int]]\
                = Lyric_line_content.extend_pronunciation_list(
                each_line_CJKV_index,
                each_line_pronunciation_list,
                character_number_each_line
            )

            # 加到输出列表
            output_list.append(each_line_pronunciation_full_list)

        return output_list

    @staticmethod
    def split_kana_tag_to_characters(kana_tag_content: str) -> list[list[str, int]]:
        # 第一个必须是数字，表示接下来几个字符是对应该读音
        # 分解kana标签
        kana_tag_list: list[list[str, int]] = []

        if kana_tag_content is None:
            return kana_tag_list

        # 第一个必须是数字，表示接下来几个字符是对应该读音
        elif kana_tag_content[0].isdigit():
            # 否则报错
            raise ValueError("The first character of kana tag must be a number representing the number of characters "
                             " corresponding to the reading.")

        else:
            # 匹配每个读音
            # 格式：[长度][读音（非数字）]
            matched_pronunciation_groups: Iterator[Match[str | bytes | Any]] = \
                Lyric_file.EACH_PROUNUNCIATION_GROUP_IN_KANA_REGEX.finditer(kana_tag_content)

            for each_matched_pronunciation_group in matched_pronunciation_groups:
                # [读音，长度]
                each_small_list: list[str, int] = [each_matched_pronunciation_group.group("pronunciation"),
                                                   each_matched_pronunciation_group.group("length")]
                kana_tag_list.append(each_small_list)

        return kana_tag_list


    def get_total_kana_tag(self) -> str:

        # 输出字符串
        output_list: str = ""

        for each_line in self.lrc_lines_secondary:
            each_line: Lyric_line
            # 调用每行的方法，得到每行的读音
            kana_tag_each_line = each_line.get_kana_tag()

            # 加入每行的读音
            output_list += kana_tag_each_line

        return output_list

    def update_pronunciation(self, pronunciation_each_line: list[list[list[Lyric_line_content, int]]]) -> Self:
        # 逐行更新
        for each_line, each_line_pronunciation in zip(self.lrc_lines_secondary, pronunciation_each_line):
            each_line: Lyric_line
            each_line_pronunciation: list[list[Lyric_line_content, int]]

            # 更新
            each_line.update_pronunciation(each_line_pronunciation)

        return self

    '''
    转为srt字幕
    '''
    def _to_srt(self) -> str:
        pass



if __name__ == '__main__':
    # 测试
    # 读取文件

    with open("../../Test_Files/ブルーバード (青鸟) - 生物股长 (いきものがかり).lrc", mode="r", encoding="utf-8") as f:
        content = f.read()
        print(content)

    a = Lyric_file.LRC_CONTENT_REGEX.match('[00:00.00][10:12:22]ブルーバード - 生物股长 (いきものがかり)')

    print(b := a.group("time_tab"))
    c = Lyric_Time_tab.TIME_TAB_EACH_LINE_VERY_LOOSE_REGREX.match(b)
    print(c)

    Lyric_file_Test_File_青鸟 = Lyric_file(content)

    print(Lyric_file_Test_File_青鸟.judge_standard_form())
    Lyric_file_Test_File_青鸟.combine_lyric_separated_to_continuous_lines()
    a = Lyric_file_Test_File_青鸟.lrc_lines_secondary
    # print(Lyric_file_Test_File_青鸟.format_output(len_of_millisecond_output=2))
    b = Lyric_file_Test_File_青鸟.compress_time_tab()
    print(Lyric_file_Test_File_青鸟.format_output(len_of_millisecond_output=2))

    print("\n\n\n\n\n")
    c = Lyric_file_Test_File_青鸟.decompress_time_tab()
    print(Lyric_file_Test_File_青鸟.format_output(len_of_millisecond_output=2))

    kana_tag = Lyric_file_Test_File_青鸟.kana
    print(kana_tag)
    print(matched_kana := Lyric_file.EACH_PROUNUNCIATION_GROUP_IN_KANA_REGEX.finditer(kana_tag))
    for i in matched_kana:
        print(i.groups())



from .Lyric_character import Lyric_character
from .Lyric_file import Lyric_file
from .Lyric_line import Lyric_line
from .Lyric_line_content import Lyric_line_content
from .Lyric_Time_tab import Lyric_Time_tab

name = "MyLyric"

__version__ = "3.0.0"
__description__ = "A simple light Lyric File processing module"
__author__ = "Ding Zedong"
__author_email__ = "2701690963@qq.com"
__all__ = ["Lyric_file", "Lyric_line", "Lyric_Time_tab", "Lyric_line_content", "Lyric_character"]



from typing import Optional, Self
from .Lyric_Time_tab import Lyric_Time_tab
from .Lyric_line_content import Lyric_line_content


class Lyric_line:

    def __init__(self,
                 time_tab_list: Optional[list[Lyric_Time_tab]],
                 lyric_content_list: list[Lyric_line_content]):

        # 歌词标签，用于适应多行合并的歌词
        # 自动排序
        self.time_tabs: Optional[list[Lyric_Time_tab]] = sorted(time_tab_list)

        # 向后兼容，用列表存储
        self.lyric_contents: list[Lyric_line_content] = lyric_content_list

    def __str__(self):
        # 两个列表，一个是时间标签，一个是歌词内容
        # 各自拼接
        time_tabs_str = ""
        for time_tab in self.time_tabs:
            time_tabs_str += str(time_tab)

        lyric_contents_str = ""
        for lyric_content in self.lyric_contents:
            lyric_contents_str += str(lyric_content)

        return time_tabs_str + lyric_contents_str

    def __repr__(self):
        return self.__str__()

    def __len__(self):
        return len(self.time_tabs)

    def __getitem__(self, item):
        # 直接合并歌词列表
        lyric_contents_str = ""
        for lyric_content in self.lyric_contents:
            lyric_contents_str += str(lyric_content)

        # 加个时间头输出
        return str(self.time_tabs[item]) + lyric_contents_str

    def __setitem__(self, key, value: Lyric_Time_tab):
        self.time_tabs[key] = value

    def __delitem__(self, key):
        del self.time_tabs[key]

    def __iter__(self):
        # 返回每个时间标签+歌词内容
        for time_tab in self.time_tabs:
            lyric_contents_str = ""
            for lyric_content in self.lyric_contents:
                lyric_contents_str += str(lyric_content)

            yield time_tab + lyric_contents_str

    def __reversed__(self):
        # 返回每个时间标签+歌词内容

        for time_tab in reversed(self.time_tabs):
            lyric_contents_str = ""
            for lyric_content in self.lyric_contents:
                lyric_contents_str += str(lyric_content)

            yield time_tab + lyric_contents_str

    def __contains__(self, item: Lyric_Time_tab):
        return item in self.time_tabs

    # 大小比较，以第一个时间标签为准
    def __lt__(self, other):
        return self.time_tabs[0] < other.time_tabs[0]

    def __le__(self, other):
        return self.time_tabs[0] <= other.time_tabs[0]

    def __eq__(self, other):
        return self.time_tabs[0] == other.time_tabs[0]

    def __ne__(self, other):
        return self.time_tabs[0] != other.time_tabs[0]

    def __gt__(self, other):
        return self.time_tabs[0] > other.time_tabs[0]

    def __ge__(self, other):
        return self.time_tabs[0] >= other.time_tabs[0]

    # 不支持加减乘除，防止混乱

    # 解构，返回每个时间标签和歌词内容 的列表
    def decompress_time_tab(self) -> list[Self]:
        output_list: list[Self] = []

        # 返回每个时间标签+歌词内容 的 Lyric_line 列表
        for time_tab in self.time_tabs:
            # 用每个时间标签 和 所有的歌词内容，新建一个 Lyric_line 对象
            new_lyric_line_object: Self = Lyric_line([time_tab], self.lyric_contents)

            # 添加到列表
            output_list.append(new_lyric_line_object)

        return output_list

    # 格式化输出
    def format_output(self,
                      len_of_millisecond_output: int = 2,
                      seperator_each_line: tuple[str, str] = (":", "."),
                      seperator_inline: tuple[str, str] = (":", ".")
                      ) -> str:

        # 用于输出字符串
        output_str: str = ""

        time_tab_str: str
        lyric_content_str: str

        # 遍历时间标签
        for time_tab in self.time_tabs:
            # 用于输出字符串
            time_tab_str = time_tab.convert_to_time_tab(len_of_millisecond_output=len_of_millisecond_output,
                                                        seperator=seperator_each_line)
            output_str += time_tab_str

        for lyric_content in self.lyric_contents:
            # 用于输出字符串
            lyric_content_str = lyric_content.format_content(len_of_millisecond_output=len_of_millisecond_output,
                                                             seperator=seperator_inline)
            output_str += lyric_content_str

        return output_str

    # 判断是否是同一句歌词
    def whether_same_lyric(self,
                           other: Self
                           ) -> bool:
        # 不管时间标签
        # 只看歌词内容
        return self.lyric_contents == other.lyric_contents

    def shift_time(self,
                   minutes: int,
                   seconds: int,
                   milliseconds: int,
                   len_of_millisecond: int = 3
                   ) -> Self:
        # 时间标签整体向后移动
        # 遍历时间标签
        for time_tab in self.time_tabs:
            time_tab.shift_time(minutes=minutes,
                                seconds=seconds,
                                milliseconds=milliseconds,
                                len_of_millisecond=len_of_millisecond)

        return self

    def get_all_chinese_and_chu_nom_and_chinese_radical(self) -> list[list[str, int]]:
        # 输出列表
        output_list: list[list[str, int]] = []

        # 逐个拼接
        for each_lyric_content in self.lyric_contents:
            each_lyric_content_CJKV: list[list[str, int]] \
                = each_lyric_content.get_all_chinese_and_chu_nom_and_chinese_radical()

            output_list += each_lyric_content_CJKV

        return output_list

    def get_kana_tag(self) -> str:
        output_str = ""
        for each_lyric_line_content in self.lyric_contents:
            output_str += each_lyric_line_content.get_kana_tag()

        return output_str

    def update_pronunciation(self, pronunciation_each_line: list[list[Lyric_line_content, int]]):
        if len(self.lyric_contents) == 1:
            self.lyric_contents[0].update_pronunciation_list(pronunciation_each_line)
        else:
            raise ValueError("Multiple lyric contents in one lyric line, not supported")


if __name__ == '__main__':
    # 测试
    # 新定义几个时间标签
    time_tab1 = Lyric_Time_tab("[00:00.00]")
    time_tab2 = Lyric_Time_tab("[00:01.00]")
    time_tab3 = Lyric_Time_tab("[00:02.00]")
    time_tab4 = Lyric_Time_tab("[00:03.00]")
    time_tab5 = Lyric_Time_tab("[00:04.00]")

    # 新定义几个歌词内容
    lyric_content1 = Lyric_line_content("歌词1")

    # 打乱顺序，组成乱序列表，用于测试
    time_tabs = [time_tab1, time_tab3, time_tab2, time_tab5, time_tab4]

    # 输入时间标签和歌词内容
    lyric_line = Lyric_line(time_tabs, [lyric_content1])

    # 输出
    print(str(lyric_line))
    print(repr(lyric_line))
    print(len(lyric_line))
    print(lyric_line.decompress_time_tab())



from collections import UserString
from .Lyric_Time_tab import Lyric_Time_tab
from typing import Optional, Union

"""
每个字符的类，继承自字符串类，
包含时间和字符
"""


class Lyric_character(UserString):

    # CJKV 汉字字符集Unicode编码范围
    # 汉字和喃字的 Unicode 区间
    # 在原有的列表后面追加新的元组
    CHINESE_OR_CHU_NOM_RANGES: list[tuple[int, int]] = [
        (0x2E80, 0x2EFF),  # CJK 部首补充
        (0x2F00, 0x2FDF),  # 康熙部首
        (0x3007, 0x3007),  # 〇
        (0x3400, 0x4DBF),  # CJK 统一表意符号扩展 A
        (0x4E00, 0x9FFF),  # CJK 统一表意符号
        (0xF900, 0xFAFF),  # CJK 兼容表意符号
        (0x20000, 0x2A6DF),  # CJK 统一表意符号扩展 B
        (0x2A700, 0x2B73F),  # CJK 统一表意符号扩展 C
        (0x2B740, 0x2B81F),  # CJK 统一表意符号扩展 D
        (0x2B820, 0x2CEAF),  # CJK 统一表意符号扩展 E
        (0x2CEB0, 0x2EBEF),  # CJK 统一表意符号扩展 F
        (0xAA60, 0xAA7F),  # 喃字补充
        (0x3005, 0x3005),  # 汉字叠字符号々
        (0x303B, 0x303B),  # 汉字叠字符号〻
        (0x20120, 0x20120),  # 多字叠字符号𠄠
        (0x16FE3, 0x16FE3),  # 多字叠字符号𖿣
        (0x2E80, 0x2E80)  # ⺀
    ]

    """
    重写init，添加时间属性
    """

    def __init__(self, character: str, time_tab: Optional[Lyric_Time_tab] = None):
        self.initial_data: str = character
        super().__init__(character)

        # 时间
        # 调用Time_tab类
        self.time_tab: Optional[Lyric_Time_tab] = time_tab

    @staticmethod
    def is_chinese_or_chu_nom_or_chinese_radical_staticmethod(single_character: Optional[str]) -> bool:
        # print(single_character, type(single_character))
        if single_character is None or single_character == "":
            return False

        char_code: int = ord(single_character)  # 获取字符的 Unicode 编码
        start: int
        end: int
        for start, end in Lyric_character.CHINESE_OR_CHU_NOM_RANGES:
            if start <= char_code <= end:  # 判断字符编码是否在汉字或喃字的 Unicode 区间内
                return True  # 如果在区间内，返回 True
        return False  # 如果不在任何区间内，返回 False

    # 非静态方法
    def is_chinese_or_chu_nom_or_chinese_radical(self) -> bool:
        if self.initial_data == "":
            return False
        else:
            return Lyric_character.is_chinese_or_chu_nom_or_chinese_radical_staticmethod(self.initial_data)



# 测试
if __name__ == '__main__':
    a_time_tab = Lyric_Time_tab("<00:00.50>", "strict")

    a = Lyric_character('覗', a_time_tab)
    print(a)
    print(a.time_tab)
    print(a.data)
    print(a.is_chinese_or_chu_nom_or_chinese_radical())
    print(Lyric_character.is_chinese_or_chu_nom_or_chinese_radical_staticmethod('々'))



import re
from typing import Optional
from typing import Final
from typing import Match, Pattern
from typing import Self

# 时间标签转换常量
CONVERSION_TIME_60: int = 60
CONVERSION_TIME_1000: int = 1000
CONVERSION_TIME_100: int = 100


class Lyric_Time_tab:
    """
    中文注释： \n
    LRC 歌词格式的 时间标签类

    English Comments: \n
    LRC Lyrics Format Time Tab Class
    """

    # 尖括号正则表达式
    ANGLE_BRACKETS_REGREX: str = r'\<|\>'
    # 方括号正则表达式
    SQUARE_BRACKETS_REGREX: str = r'\[|\]'

    '''
    判断时间标签是否合法
    拥有严格，普通，宽松，非常宽松四种模式
    strict: 严格模式，只接受[00:00.00]、<00:00.00>格式
    normal: 普通模式，接受[00:00[:.]{2-3}]、<00:00[:.]{2-3}>格式
    loose: 宽松模式，在normal基础上 允许 分钟秒钟毫秒任意位数，至少一位，但是不允许括号缺失， 毫秒数可缺失 [ \d* : \d* [:.] \d* ]
    very_loose: 非常宽松模式，允许括号缺失 [? \d* : \d* [:.] \d* ]?
    
    # 规范里面说了，分钟位和秒钟位必须有数字
    '''
    # 每句歌词的时间标签的正则表达式 []
    # 严格模式时间标签的正则表达式
    TIME_TAB_EACH_LINE_STRICT_REGREX: Final[Pattern[str]] = re.compile(r'(?P<left_bracket>\[)'
                                                                       r'(?P<minutes>\d{2})'
                                                                       r'(?P<minutes_seconds_seperator>:)'
                                                                       r'(?P<seconds>\d{2})'
                                                                       r'(?P<seconds_milliseconds_seperator>\.)'
                                                                       r'(?P<milliseconds>\d{2})'
                                                                       r'(?P<right_bracket>])')
    # 普通模式时间标签的正则表达式
    TIME_TAB_EACH_LINE_NORMAL_REGREX: Final[Pattern[str]] = re.compile(r'(?P<left_bracket>\[)'
                                                                       r'(?P<minutes>\d{2})'
                                                                       r'(?P<minutes_seconds_seperator>:)'
                                                                       r'(?P<seconds>\d{2})'
                                                                       r'(?P<seconds_milliseconds_seperator>[:.])'
                                                                       r'(?P<milliseconds>\d{2,3})'
                                                                       r'(?P<right_bracket>])')

    # 宽松模式时间标签的正则表达式
    TIME_TAB_EACH_LINE_LOOSE_REGREX: Final[Pattern[str]] = re.compile(r'(?P<left_bracket>\[)'
                                                                      r'(?P<minutes>\d*)'
                                                                      r'(?P<minutes_seconds_seperator>:)'
                                                                      r'(?P<seconds>\d*)'
                                                                      r'(?P<seconds_milliseconds_seperator>[:.])?'
                                                                      r'(?P<milliseconds>\d*)?'
                                                                      r'(?P<right_bracket>])')
    # 非常宽松模式时间标签的正则表达式
    TIME_TAB_EACH_LINE_VERY_LOOSE_REGREX: Final[Pattern[str]] = re.compile(r'(?P<left_bracket>\[)?'
                                                                           r'(?P<minutes>\d*)'
                                                                           r'(?P<minutes_seconds_seperator>:)'
                                                                           r'(?P<seconds>\d*)'
                                                                           r'(?P<seconds_milliseconds_seperator>[:.])?'
                                                                           r'(?P<milliseconds>\d*)?'
                                                                           r'(?P<right_bracket>])?')

    # 正则表达式列表
    TIME_TAB_EACH_LINE_REGREX_LIST: list = [TIME_TAB_EACH_LINE_STRICT_REGREX,
                                            TIME_TAB_EACH_LINE_NORMAL_REGREX,
                                            TIME_TAB_EACH_LINE_LOOSE_REGREX,
                                            TIME_TAB_EACH_LINE_VERY_LOOSE_REGREX]

    # 歌词每个字的时间标签的正则表达式 <>
    # 严格模式时间标签的正则表达式
    TIME_TAB_EACH_WORD_STRICT_REGREX: Final[Pattern[str]] = re.compile(r'(?P<left_bracket><)'
                                                                       r'(?P<minutes>\d{2})'
                                                                       r'(?P<minutes_seconds_seperator>:)'
                                                                       r'(?P<seconds>\d{2})'
                                                                       r'(?P<seconds_milliseconds_seperator>\.)'
                                                                       r'(?P<milliseconds>\d{2})'
                                                                       r'(?P<right_bracket>>)')
    # 普通模式时间标签的正则表达式
    TIME_TAB_EACH_WORD_NORMAL_REGREX: Final[Pattern[str]] = re.compile(r'(?P<left_bracket><)'
                                                                       r'(?P<minutes>\d{2})'
                                                                       r'(?P<minutes_seconds_seperator>:)'
                                                                       r'(?P<seconds>\d{2})'
                                                                       r'(?P<seconds_milliseconds_seperator>[:.])'
                                                                       r'(?P<milliseconds>\d{2,3})'
                                                                       r'(?P<right_bracket>>)')
    # 宽松模式时间标签的正则表达式
    TIME_TAB_EACH_WORD_LOOSE_REGREX: Final[Pattern[str]] = re.compile(r'(?P<left_bracket><)'
                                                                      r'(?P<minutes>\d*)'
                                                                      r'(?P<minutes_seconds_seperator>:)'
                                                                      r'(?P<seconds>\d*)'
                                                                      r'(?P<seconds_milliseconds_seperator>[:.])?'
                                                                      r'(?P<milliseconds>\d*)?'
                                                                      r'(?P<right_bracket>>)')
    # 非常宽松模式时间标签的正则表达式
    TIME_TAB_EACH_WORD_VERY_LOOSE_REGREX: Final[Pattern[str]] = re.compile(r'(?P<left_bracket><)?'
                                                                           r'(?P<minutes>\d*)'
                                                                           r'(?P<minutes_seconds_seperator>:)'
                                                                           r'(?P<seconds>\d*)'
                                                                           r'(?P<seconds_milliseconds_seperator>[:.])?'
                                                                           r'(?P<milliseconds>\d*)?'
                                                                           r'(?P<right_bracket>>)?')

    # 正则表达式列表
    TIME_TAB_EACH_WORD_REGREX_LIST: list = [TIME_TAB_EACH_WORD_STRICT_REGREX,
                                            TIME_TAB_EACH_WORD_NORMAL_REGREX,
                                            TIME_TAB_EACH_WORD_LOOSE_REGREX,
                                            TIME_TAB_EACH_WORD_VERY_LOOSE_REGREX]

    """
    接受一个时间标签字符串，分离出时间标签的各个部分
    """

    def __init__(self, tab: Optional[str], mode: str = 'normal'):
        # 时间标签原始字符串
        self.original_time_tab: str = tab
        # 修改或者规范后的时间标签字符串
        self.time_tab: Optional[str] = None

        # 时间标签类型 [] or <>
        self.brackets: Optional[list[str, str]] = None
        # 时间戳
        self.time_stamp: Optional[float] = None

        # 原始匹配结果
        self.match_result: Optional[Match[str]] = None
        # 时间标签列表
        self.time_list: Optional[list] = None

        # 时间标签分钟
        self.minutes_str: Optional[str] = None
        # 时间标签秒
        self.seconds_str: Optional[str] = None
        # 时间标签毫秒（不足三位补零）（默认按照三位储存，输出则是默认两位）
        self.milliseconds_str: Optional[str] = None

        # 分钟秒钟分割符
        self.minutes_seconds_seperator: Optional[str] = None
        # 秒钟毫秒分割符
        self.seconds_milliseconds_seperator: Optional[str] = None

        # 时间标签毫秒位数
        self.len_of_millisecond = None

        # 预分离时间标签
        if tab is not None:
            self.__pre_separating(tab, mode)
        else:
            # print("Time tab None input, notice!")
            pass

        # 最后初始化 self.time_tab， 为 self.original_time_tab
        self.time_tab = self.original_time_tab

    # 返回时间标签时间戳
    def __int__(self):
        return self.time_stamp

    # 返回时间标签字符串
    def __str__(self):
        return self.time_tab

    # 返回时间标签列表
    def __repr__(self):
        return self.time_list

    # 返回时间标签是否 == 另一个时间标签
    def __eq__(self, other):
        return self.time_stamp == other.time_stamp

    # 返回时间标签是否 < 于另一个时间标签
    def __lt__(self, other):
        return self.time_stamp < other.time_stamp

    # 返回时间标签是否 <= 另一个时间标签
    def __le__(self, other):
        return self.time_stamp <= other.time_stamp

    # 返回时间标签是否 != 另一个时间标签
    def __ne__(self, other):
        return self.time_stamp != other.time_stamp

    # 返回时间标签是否 > 另一个时间标签
    def __gt__(self, other):
        return self.time_stamp > other.time_stamp

    # 返回时间标签是否 >= 另一个时间标签
    def __ge__(self, other):
        return self.time_stamp >= other.time_stamp

    # 加减乘除运算，返回时间标签时间戳
    def __add__(self, other):
        # 判断类型是否为 时间标签
        if isinstance(other, self.__class__):
            self.time_stamp += other.time_stamp
            return self
        # 判断类型是否为 int 或者 float
        elif isinstance(other, int) or isinstance(other, float):
            self.time_stamp += other * CONVERSION_TIME_1000
            return self
        else:
            raise TypeError('unsupported operand type(s) for +: \'TimeTab\' and \'{}\''.format(type(other)))

    def __sub__(self, other):
        # 判断类型是否为 时间标签
        if isinstance(other, self.__class__):
            self.time_stamp -= other.time_stamp
            return self
        # 判断类型是否为 int 或者 float
        elif isinstance(other, int) or isinstance(other, float):
            self.time_stamp -= other * CONVERSION_TIME_1000
            return self
        else:
            raise TypeError('unsupported operand type(s) for -: \'TimeTab\' and \'{}\''.format(type(other)))

    def __mul__(self, other):
        # 判断类型是否为 时间标签
        if isinstance(other, self.__class__):
            self.time_stamp *= other.time_stamp
            return self
        # 判断类型是否为 int 或者 float
        elif isinstance(other, int) or isinstance(other, float):
            self.time_stamp *= other * CONVERSION_TIME_1000
            return self
        else:
            raise TypeError('unsupported operand type(s) for *: \'TimeTab\' and \'{}\''.format(type(other)))

    def __truediv__(self, other):
        # 判断类型是否为 时间标签
        if isinstance(other, self.__class__):
            self.time_stamp /= other.time_stamp
            return self
        # 判断类型是否为 int 或者 float
        elif isinstance(other, int) or isinstance(other, float):
            self.time_stamp /= other * CONVERSION_TIME_1000
            return self
        else:
            raise TypeError('unsupported operand type(s) for /: \'TimeTab\' and \'{}\''.format(type(other)))

    def __floordiv__(self, other):
        # 判断类型是否为 时间标签
        if isinstance(other, self.__class__):
            self.time_stamp //= other.time_stamp
            return self
        # 判断类型是否为 int 或者 float
        elif isinstance(other, int) or isinstance(other, float):
            self.time_stamp //= other * CONVERSION_TIME_1000
            return self
        else:
            raise TypeError('unsupported operand type(s) for //: \'TimeTab\' and \'{}\''.format(type(other)))

    def __mod__(self, other):
        # 判断类型是否为 时间标签
        if isinstance(other, self.__class__):
            self.time_stamp %= other.time_stamp
            return self
        # 判断类型是否为 int 或者 float
        elif isinstance(other, int) or isinstance(other, float):
            self.time_stamp %= other * CONVERSION_TIME_1000
            return self
        else:
            raise TypeError('unsupported operand type(s) for %: \'TimeTab\' and \'{}\''.format(type(other)))

    def __pow__(self, other):
        # 判断类型是否为 时间标签
        if isinstance(other, self.__class__):
            self.time_stamp **= other.time_stamp
            return self
        # 判断类型是否为 int 或者 float
        elif isinstance(other, int) or isinstance(other, float):
            self.time_stamp **= other * CONVERSION_TIME_1000
            return self
        else:
            raise TypeError('unsupported operand type(s) for **: \'TimeTab\' and \'{}\''.format(type(other)))

    def __radd__(self, other):
        return self.__add__(other)

    def __rsub__(self, other):
        return self.__sub__(other)

    def __rmul__(self, other):
        return self.__mul__(other)

    def __rtruediv__(self, other):
        return self.__truediv__(other)

    def __rfloordiv__(self, other):
        return self.__floordiv__(other)

    def __rmod__(self, other):
        return self.__mod__(other)

    def __rpow__(self, other):
        return self.__pow__(other)

    """
    预分离标签，判断是否合法，分离出时间标签的各个部分，储存到类的属性中，供其他方法调用
    私有方法
    """

    def __pre_separating(self, tab: str, mode: str = 'normal') -> None:
        """
        中文注释：
        预分离标签，判断是否合法，分离出时间标签的各个部分，储存到类的属性中，供其他方法调用
        私有方法

        English comment:
        Pre-separate the label, judge whether it is legal, separate the various parts of the time label,
        and store them in the properties of the class for other methods to call
        Private method

        :param tab: 时间标签字符串
        :param mode: 模式
        :return: None
        """

        # 匹配时间标签
        # 判断是否合法，并且判断类型
        valid_with_type: list[bool, Optional[Pattern[str]]] = Lyric_Time_tab.is_valid_with_type(tab, mode)

        # 如果合法
        if valid_with_type[0]:
            # 利用返回的 判断出的 正则表达式 分离时间标签
            self.match_result = valid_with_type[1].match(tab)
            self.time_list = self.match_result.groups()

            # 添加到类的属性中
            self.brackets = self.match_result.group("left_bracket") + self.match_result.group("right_bracket")
            self.minutes_str = self.match_result.group("minutes")
            # 如果有分钟位，分钟位不足两位，左边补零
            self.minutes_str = self.minutes_str.rjust(2, '0')
            self.minutes_seconds_seperator = self.match_result.group("minutes_seconds_seperator")

            self.seconds_str = self.match_result.group("seconds")
            # 如果有秒位，秒位不足两位，左边补零
            self.seconds_str = self.seconds_str.rjust(2, '0')

            self.seconds_milliseconds_seperator = self.match_result.group("seconds_milliseconds_seperator")
            # 如果有毫秒位，毫秒位不足三位，右边补零
            self.milliseconds_str = self.match_result.group("milliseconds")

            # 如果有毫秒位，毫秒位不足三位，右边补零
            if self.milliseconds_str is not None:
                self.len_of_millisecond = len(self.milliseconds_str)
                self.milliseconds_str = self.milliseconds_str.ljust(3, '0')
            else:
                # 原先就是None，不用管
                pass

            # 自动计算时间戳
            # 要考虑没有(None)的情况
            # None 会被转换成 0
            minutes_int: int
            seconds_int: int
            milliseconds_int: int

            if self.minutes_str is not None:
                minutes_int = int(self.minutes_str)
            else:
                minutes_int = 0

            if self.seconds_str is not None:
                seconds_int = int(self.seconds_str)
            else:
                seconds_int = 0

            if self.milliseconds_str is not None:
                milliseconds_int = int(self.milliseconds_str)
            else:
                milliseconds_int = 0

            # 调用函数，计算时间戳
            self.time_stamp = Lyric_Time_tab.calculate_time_stamp(minutes_int, seconds_int, milliseconds_int)

            # print(self.time_stamp)

        # 如果不合法
        else:
            # 报错
            raise ValueError(f"The time original_tab {tab} is not valid under {mode} mode")

    """
    判断时间标签是否合法，并且判断类型
    """

    @classmethod
    def is_valid_with_type(cls, tab: str, mode: str = 'normal') -> list[bool, Optional[Pattern[str]]]:
        """
        中文： \n
        判断时间标签是否合法，并且判断类型 \n
        拥有严格，普通，宽松，非常宽松四种模式 \n
            strict: 严格模式，只接受[00:00.00]、<00:00.00>格式 \n
            normal: 普通模式，接受[00:00[:.]{2-3}]、<00:00[:.]{2-3}>格式 \n
            loose: 宽松模式，在normal基础上 允许 分钟秒钟毫秒任意位数，但是不允许括号缺失， 毫秒数可缺失 \n
            very_loose: 非常宽松模式，允许括号缺失 \n

        可以是 每行的也可以是每个字的 [] <> \n
            如果是每行的，返回[True, 对应的模式下的 方括号正则表达式常量] \n
            如果是每个字的，返回[True, 对应的模式下的 尖括号正则表达式常量] \n
            如果不是任何一种，返回[False, None]
        \n\n

        English: \n
        Determine whether the time label is legal and determine the type \n
        Has strict, normal, loose, very loose four modes \n
            strict: strict mode, only accept [00:00.00], <00:00.00> format \n
            normal: normal mode, accept [00:00[:.]{2-3}], <00:00[:.]{2-3}> format \n
            loose: loose mode, on the basis of normal,
                allow minutes_str, seconds_str, milliseconds_str any number of digits,
            but do not allow brackets missing, milliseconds_str can be missing \n
            very_loose: very loose mode, allow brackets missing \n

        Can be every line or every word [] <> \n
            If it is every line, return [True, square brackets regular expression constant] \n
            If it is every word, return [True, angle brackets regular expression constant] \n
            If it is not any one, return [False, None] \n

        :param tab: 时间标签 Time Tab
        :param mode: 模式 Mode
        :return: [是否合法 Whether valid,
                  括号正则表达式常量 Bracket Regular Expression Constant]
        """

        # 严格模式
        if mode == 'strict':
            # 每行的
            if re.match(cls.TIME_TAB_EACH_LINE_STRICT_REGREX, tab):
                return [True, Lyric_Time_tab.TIME_TAB_EACH_LINE_STRICT_REGREX]
            # 每个字的
            elif re.match(cls.TIME_TAB_EACH_WORD_STRICT_REGREX, tab):
                return [True, cls.TIME_TAB_EACH_WORD_STRICT_REGREX]
            else:
                return [False, None]

        # 普通模式
        elif mode == 'normal':
            # 每行的
            if re.match(cls.TIME_TAB_EACH_LINE_NORMAL_REGREX, tab):
                return [True, cls.TIME_TAB_EACH_LINE_NORMAL_REGREX]
            # 每个字的
            elif re.match(cls.TIME_TAB_EACH_WORD_NORMAL_REGREX, tab):
                return [True, cls.TIME_TAB_EACH_WORD_NORMAL_REGREX]
            else:
                return [False, None]

        # 宽松模式
        elif mode == 'loose':
            # 每行的
            if re.match(cls.TIME_TAB_EACH_LINE_LOOSE_REGREX, tab):
                return [True, cls.TIME_TAB_EACH_LINE_LOOSE_REGREX]
            # 每个字的
            elif re.match(cls.TIME_TAB_EACH_WORD_LOOSE_REGREX, tab):
                return [True, cls.TIME_TAB_EACH_WORD_LOOSE_REGREX]
            else:
                return [False, None]

        # 非常宽松模式
        elif mode == 'very_loose':
            # 每行的
            if re.match(cls.TIME_TAB_EACH_LINE_VERY_LOOSE_REGREX, tab):
                return [True, cls.TIME_TAB_EACH_LINE_VERY_LOOSE_REGREX]
            # 每个字的
            elif re.match(cls.TIME_TAB_EACH_WORD_VERY_LOOSE_REGREX, tab):
                return [True, cls.TIME_TAB_EACH_WORD_VERY_LOOSE_REGREX]
            else:
                return [False, None]

        # 模式错误
        else:
            # 引发异常, 模式错误
            # input is "mode变量的值", must be strict, normal, loose or very_loose
            raise ValueError(f'input is "{mode}", must be strict, normal, loose or very_loose')

    """
    计算时间戳，分、秒、毫秒，小时（可选）
    返回毫秒位单位的时间戳(3位)
    """

    @staticmethod
    def calculate_time_stamp(minute: int, second: int, millisecond: float, hour: int = 0) -> float:
        """
        中文：\n
        计算时间戳，分、秒、毫秒，小时（可选）\n
        返回毫秒位单位的时间戳(3位)

        English: \n
        Calculate the time stamp, minute, second, millisecond, hour (optional) \n
        Return the time stamp in milliseconds_str (3 digits)

        :param minute: 分 Minute
        :param second: 秒 Second
        :param millisecond: 毫秒 Millisecond
        :param hour: 小时 Hour
        :return: 毫秒位单位的时间戳(3位) The time stamp in milliseconds_str (3 digits)
        """

        # 括号不影响，不会转为元组
        return (hour * CONVERSION_TIME_60 * CONVERSION_TIME_60 * CONVERSION_TIME_1000 +
                minute * CONVERSION_TIME_60 * CONVERSION_TIME_1000 +
                second * CONVERSION_TIME_1000 +
                millisecond)

    """
    规范时间戳，按需求转为len_of_millisecond位的时间戳
    """

    @staticmethod
    def format_time_stamp_static(time_stamp: int, len_of_millisecond: int = 2, keep_decimal: bool = False) -> int:
        """
        中文：\n
        默认是三位， \n
        转为len_of_millisecond位的时间戳 \n
        比如，2位的时间戳，就是除以10

        English: \n
        Default is three digits, \n
        Convert to a time stamp of len_of_millisecond digits \n
        For example, a 2-digit time stamp is divided by 10

        :param time_stamp: 时间戳 The time stamp
        :param len_of_millisecond: 毫秒位的位数 The number of milliseconds_str
        :param keep_decimal: 是否保留小数位 Whether to keep decimal places
        """

        time_stamp: int | float

        # 计算时间戳
        if len_of_millisecond == 3:
            time_stamp = time_stamp
        else:
            time_stamp = time_stamp / (10 ** (3 - len_of_millisecond))

        # 是否保留小数位
        if keep_decimal:
            return time_stamp
        else:
            return int(time_stamp)

    """
    规范时间戳，按需求转为len_of_millisecond位的时间戳
    """

    def format_time_stamp(self, len_of_millisecond: int = 2, keep_decimal: bool = False) -> int:
        """
        中文：\n
        用静态方法 规范类为时间戳

        English: \n
        Use static method to standardize class as time stamp

        :param len_of_millisecond: 毫秒位的位数 The number of milliseconds_str
        :param keep_decimal: 是否保留小数位 Whether to keep decimal places
        """
        return Lyric_Time_tab.format_time_stamp_static(self.time_stamp, len_of_millisecond, keep_decimal)

    """
    将时间戳转为时间标签
    """

    @staticmethod
    def convert_time_stamp_to_time_tab_static(time_stamp: int | float,
                                              len_of_millisecond_inputted: int = 3,
                                              len_of_millisecond_output: int = 2,
                                              brackets: tuple[str, str] = ("[", "]"),
                                              seperator: tuple[str, str] = (":", ".")) -> str:
        """
        中文：\n
        将时间戳转为时间标签

        English: \n
        It is a staticmethod. It converts the time stamp of time tag in to a tag.

        :param time_stamp: 时间戳 The time stamp
        :param len_of_millisecond_inputted: 输入的时间戳的毫秒位的位数 The number of milliseconds_str of the input time stamp
        :param len_of_millisecond_output: 输出的时间戳的毫秒位的位数 The number of milliseconds_str of the output time stamp
        :param brackets: 括号 The brackets
        :param seperator: 分隔符 The seperator
        :return: 时间标签 The time tag
        """
        minutes_int: int
        seconds_int: int
        millisecond_int: int | float

        minutes_str: str
        seconds_str: str
        millisecond_str: str

        time_tab_output: str

        # 计算分秒毫秒，输入的时间戳是len_of_millisecond位相关的
        minutes_int = time_stamp // (10 ** len_of_millisecond_inputted) // 60
        seconds_int = time_stamp // (10 ** len_of_millisecond_inputted) % 60
        millisecond_int = time_stamp * (10 ** (3 - len_of_millisecond_inputted)) % 1000

        # 转为字符串
        # 分
        minutes_str = str(minutes_int)
        # 补位
        # 不足则左边补0
        minutes_str = minutes_str.rjust(2, "0")

        # 秒
        seconds_str = str(seconds_int)
        # 补位
        # 不足则左边补0
        seconds_str = seconds_str.rjust(2, "0")

        # 毫秒
        # 如果有小数位，抹去小数位
        millisecond_int = int(millisecond_int)
        # 转为字符串
        millisecond_str = str(millisecond_int)
        # 输出的毫秒位长度
        # 不足则右边补0
        millisecond_str = millisecond_str.ljust(len_of_millisecond_output, "0")
        # 截取
        millisecond_str = millisecond_str[:len_of_millisecond_output]

        # 加上 左右括号 和 分隔符
        # 格式化字符串
        time_tab_output = f"{brackets[0]}" \
                          f"{minutes_str}" \
                          f"{seperator[0]}" \
                          f"{seconds_str}" \
                          f"{seperator[1]}" \
                          f"{millisecond_str}" \
                          f"{brackets[1]}"

        # 返回最终结果
        return time_tab_output

    def convert_to_time_tab(self,
                            len_of_millisecond_inputted: int = 3,
                            len_of_millisecond_output: int = 2,
                            brackets: tuple[str, str] = ("[", "]"),
                            seperator: tuple[str, str] = (":", ".")) -> str:
        """
        中文：\n
        将时间戳转为时间标签，对实例本身进行操作

        English: \n
        It converts the time stamp of time tag in to a tag. It operates on the instance itself.

        :param len_of_millisecond_inputted: 输入的时间戳的毫秒位的位数 The number of milliseconds_str of the input time stamp
            默认在实例化类的时候已经转换成了3位毫秒位的时间戳 Default is 3 digits in property time_stamp of class Lyric_Time_tab
        :param len_of_millisecond_output: 输出的时间戳的毫秒位的位数 The number of milliseconds_str of the output time stamp
        :param brackets: 括号 The brackets
        :param seperator: 分隔符 The seperator
        :return: 时间标签 The time tag
        """
        # 如果time stamp是None，返回空字符串，表明没有时间标签
        if self.time_stamp is None:
            return ""
        else:
            return Lyric_Time_tab.convert_time_stamp_to_time_tab_static(self.time_stamp,
                                                                        len_of_millisecond_inputted,
                                                                        len_of_millisecond_output,
                                                                        brackets,
                                                                        seperator)

    # 返回自身
    def shift_time(self,
                   minutes: int,
                   seconds: int,
                   milliseconds: int,
                   len_of_millisecond: int = 3
                   ) -> Self:
        """
        中文：\n
        将时间标签向前或向后移动

        English: \n
        Move the time tag forward or backward

        :param minutes: 分钟数 The number of minutes_str
        :param seconds: 秒数 The number of seconds_str
        :param milliseconds: 毫秒数 The number of milliseconds_str
        :param len_of_millisecond: 毫秒位的位数 The number of milliseconds_str
        """

        # 转毫秒位为3位（规范化）
        milliseconds = int(milliseconds * (10 ** (3 - len_of_millisecond)))

        # 修改属性
        self.minutes_str += minutes
        self.seconds_str += seconds
        self.milliseconds_str += milliseconds

        # 修改时间列表
        self.time_list[1] += minutes
        self.time_list[3] += seconds
        self.time_list[5] += milliseconds

        # 计算时间戳
        time_stamp_shift: float = Lyric_Time_tab.calculate_time_stamp(minutes, seconds, milliseconds)

        # 移动时间戳
        self.time_stamp += time_stamp_shift

        # 修改时间标签
        self.time_tab = self.convert_to_time_tab()

        return self

    # 返回自身
    def format_time_tab_self(self,
                             brackets: Optional[tuple[str, str]],
                             seperator: Optional[tuple[str, str]]
                             ) -> Self:
        """
        中文：\n
        格式化时间标签对象本身 \n
        把秒 限制在0-59之间 \n
        把毫秒 限制在0-999之间 \n
        并且补全括号，分隔符 \n
        如果brackets和seperator为None，则不补全括号，分隔符
        返回规范化后的自身

        English: \n
        Format the time tag object itself\n
        Limit the seconds_str between 0 and 59 \n
        Limit the milliseconds_str between 0 and 999 \n
        And complete the brackets and seperator \n
        If brackets and seperator are None, do not complete the brackets and seperator
        Return the normalized self


        :return: self
        """

        # ==================== 时分秒毫秒单位溢出处理 ==================== #

        # 预处理 类型转换
        # 转为float
        minutes: int = int(self.minutes_str)
        seconds: int = int(self.seconds_str)
        milliseconds: float = float(self.milliseconds_str)

        # 毫秒
        # 如果大于等于1000
        if milliseconds >= 1000:
            # 计算多余的秒数
            seconds_extra: int = int(milliseconds // 1000)
            # 计算剩余的毫秒数
            milliseconds = milliseconds % 1000
            # 秒数加上多余的秒数
            seconds += seconds_extra

        # 如果小于0
        elif milliseconds < 0:
            # 注意这里 用的是负数相加，milliseconds是负数，所以退位减一是负一
            # 计算多余的秒数
            seconds_extra: int = -1 + int(milliseconds // 1000)
            # 计算剩余的毫秒数
            milliseconds = 1000 + milliseconds % 1000
            # 秒数加上多余的秒数
            seconds += seconds_extra

        # 秒
        # 如果大于等于60
        if seconds >= 60:
            # 计算多余的分钟数
            minutes_extra: int = int(seconds // 60)
            # 计算剩余的秒数
            seconds = seconds % 60
            # 分钟数加上多余的分钟数
            minutes += minutes_extra

        # 如果小于0
        elif seconds < 0:
            # 注意这里 用的是负数相加，seconds是负数，所以退位减一是负一
            # 计算多余的分钟数
            minutes_extra: int = -1 + int(seconds // 60)
            # 计算剩余的秒数
            seconds = 60 + seconds % 60
            # 分钟数加上多余的分钟数
            minutes += minutes_extra

        # 赋值回去
        self.minutes_str = str(minutes)
        self.seconds_str = str(seconds)
        self.milliseconds_str = str(milliseconds)

        # ==================== 括号分隔符补全 ==================== #

        # 如果brackets和seperator为None，则不补全括号，分隔符
        if brackets is not None:
            self.brackets = brackets

            # 也赋值到时间列表内
            self.time_list[0] = self.brackets[0]
            self.time_list[6] = self.brackets[1]

        if seperator is not None:
            self.minutes_seconds_seperator = seperator[0]
            self.seconds_milliseconds_seperator = seperator[1]

            # 也赋值到时间列表内
            self.time_list[2] = self.minutes_seconds_seperator
            self.time_list[4] = self.seconds_milliseconds_seperator

        # ==================== 修改时间列表内的分秒毫秒 ==================== #
        self.time_list[1] = minutes
        self.time_list[3] = seconds
        self.time_list[5] = milliseconds

        return self

    def isspace(self) -> bool:
        return self.original_time_tab.isspace()


"""
中文：\n
测试内容

English: \n
Test content
"""
if __name__ == '__main__':
    # 打印正则表达式列表
    for i in Lyric_Time_tab.TIME_TAB_EACH_LINE_REGREX_LIST:
        print(i)

    print(Lyric_Time_tab.TIME_TAB_EACH_LINE_NORMAL_REGREX.pattern)

    print(Lyric_Time_tab.calculate_time_stamp(10, 1, 100))

    # 测试时间标签 += 运算符
    time_tab = Lyric_Time_tab("[00:00.00]")
    time_tab += 1
    print(time_tab.convert_to_time_tab())
